;;; Copyright (c) 2019 David Ireland.  All rights reserved.
;;; BSD 3-Clause License: http://opensource.org/licenses/BSD-3-Clause

(import
  scheme
  srfi-1
  srfi-13
  srfi-69
  chicken.io
  chicken.load
  chicken.base
  chicken.port
  chicken.file
  chicken.format
  chicken.irregex
  chicken.condition
  chicken.process-context)

;;; K-Lambda 
(include "klambda.scm")

;;; Shen
(include "shen-chicken/toplevel.kl.scm")
(include "shen-chicken/core.kl.scm")
(include "shen-chicken/sys.kl.scm")
(include "shen-chicken/sequent.kl.scm")
(include "shen-chicken/yacc.kl.scm")
(include "shen-chicken/reader.kl.scm")
(include "shen-chicken/prolog.kl.scm")
(include "shen-chicken/track.kl.scm")
(include "shen-chicken/load.kl.scm")
(include "shen-chicken/writer.kl.scm")
(include "shen-chicken/macros.kl.scm")
(include "shen-chicken/dict.kl.scm")
(include "shen-chicken/declarations.kl.scm")
(include "shen-chicken/types.kl.scm")
(include "shen-chicken/t-star.kl.scm")

;;;  ___       _                        _
;;; |_ _|_ __ | |_ ___ _ __ _ __   __ _| |
;;;  | || '_ \| __/ _ \ '__| '_ \ / _` | |
;;;  | || | | | ||  __/ |  | | | | (_| | |
;;; |___|_| |_|\__\___|_|  |_| |_|\__,_|_|
;;;
;;;   ___                               _ _
;;;  / _ \__   _____ _ ____      ___ __(_) |_ ___  ___
;;; | | | \ \ / / _ \ '__\ \ /\ / / '__| | __/ _ \/ __|
;;; | |_| |\ V /  __/ |   \ V  V /| |  | | ||  __/\__ \
;;;  \___/  \_/ \___|_|    \_/\_/ |_|  |_|\__\___||___/

;;; Overwrites interal Shen procedures for better performance.

(define (kl:map V2961 V2962)
  (map V2961 V2962))

(define (kl:append V2735 V2736)
  (append V2735 V2736))

(define (kl:sum V2899)
  (apply + V2899))

(define (kl:reverse V2925)
  (reverse V2925))

(define (kl:cd path)
  (if (string=? path "")
    (let ((path (kl:value (quote shen.*initial-home-directory*))))
      (change-directory path)
      (kl:set (quote *home-directory*) path))
    (begin
      (change-directory path)
      (kl:set (quote *home-directory*) path))))

;;;     _    ____ ___
;;;    / \  |  _ \_ _|
;;;   / _ \ | |_) | |
;;;  / ___ \|  __/| |
;;; /_/   \_\_|  |___|

(define (read-file file)
  (map
   (lambda (x)
     (kl:shen.eval-without-macros x))
   (kl:read-file file)))

(define (read-from-string str)
  (let ((shen (with-input-from-string str read)))
    (eval-without-macros shen)))

(define (eval-without-macros x)
  (kl:shen.eval-without-macros x))

(define (run-shen)
  (kl:shen.shen))




